# -*- coding: utf-8 -*-
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django_extensions.db import fields as exfields

class BaseSemLink(models.Model):
    """Linked Data generic link model, can apply to SeeAlso or SameAs"""
    url             = models.URLField(blank=True, verify_exists=True, verbose_name=_(u'URL'))
    label           = models.CharField(_(u'Label'),max_length=100)
    slug            = exfields.AutoSlugField(populate_from=('label'))
    created         = exfields.CreationDateTimeField(_(u'created'),null=True)
    modified        = exfields.ModificationDateTimeField(_(u'modified'),null=True)
    content_type    = models.ForeignKey(ContentType, blank=True, null=True)
    object_id       = models.PositiveIntegerField()
    content_object  = generic.GenericForeignKey('content_type', 'object_id')
    
    # TODO et author uri etc ?
    
    def __unicode__(self):
        return self.label
    class Meta :
        abstract = True
        
        




# -*- coding:utf-8 -*-

from django.shortcuts import render_to_response, redirect
from coop_local.models import Organization, Person
from django.template import RequestContext
from django.core.urlresolvers import reverse
from django.conf import settings

def home(request):
    rdict = {}
    rdict['dernieres_initiatives'] = Organization.objects.filter(active=True)[:10]
    rdict['inscrits'] = Person.objects.all().count()
    return render_to_response('home.html',rdict,RequestContext(request))


"""
This file was generated with the customdashboard management command, it
contains the two classes for the main dashboard and app index dashboard.
You can customize these classes as you want.

To activate your index dashboard add the following to your settings.py::
    ADMIN_TOOLS_INDEX_DASHBOARD = 'devcoop.dashboard.CustomIndexDashboard'

And to activate the app index dashboard::
    ADMIN_TOOLS_APP_INDEX_DASHBOARD = 'devcoop.dashboard.CustomAppIndexDashboard'
"""

from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse

from admin_tools.dashboard import modules, Dashboard, AppIndexDashboard
from admin_tools.utils import get_admin_site_name


class CustomIndexDashboard(Dashboard):
    """
    Custom index dashboard for devcoop.
    """
    columns = 2
    title = ''
    template = 'admin_tools/coop_dashboard.html'

    def init_with_context(self, context):
        site_name = get_admin_site_name(context)
        # append a link list module for "quick links"
        self.children.append(modules.LinkList(
            _('Quick links'),
            layout='inline',
            draggable=False,
            deletable=False,
            collapsible=False,
            children=[
                [_('Return to site'), '/'],
                [_('Change password'),
                 reverse('%s:password_change' % site_name)],
                [_('Log out'), reverse('%s:logout' % site_name)],
            ]
        ))

        self.children.append(modules.AppList(
            _(u"Navigation and articles"),
            models=('coop_cms.models.NavTree',
                    'coop_local.models.Article',
                    #'coop_cms.rss_sync.models.RssItem',
                    'django.contrib.comments.Comment',
                    ),
            template='admin_tools/coop_applist.html'
        ))
        
        self.children.append(modules.AppList(
            _(u"My network"),
            models=('coop_agenda.models.Event',
                    'coop_agenda.models.EventType',
                    'coop_local.models.Person',
                    'coop_local.models.PersonCategory',
                    'coop_local.models.Organization',
                    'coop_local.models.OrganizationCategory',
                    'coop_local.models.Ctag'
                    ),
            template='admin_tools/coop_applist.html',
        ))
        

        # append an app list module for "Administration"
        self.children.append(modules.AppList(
            _('Administration'),
            models=('django.contrib.*',),
            template='admin_tools/coop_applist.html',
        ))

        # append a recent actions module
        self.children.append(modules.RecentActions(_('Recent Actions'), 5))

        # append a feed module
        self.children.append(modules.Feed(
            _('Latest Django News'),
            feed_url='http://www.djangoproject.com/rss/weblog/',
            limit=5
        ))

        # append another link list module for "support".
        self.children.append(modules.LinkList(
            _('Support'),
            children=[
                {
                    'title': _('Blog PES'),
                    'url': 'http://blog.credis.org/',
                    'external': True,
                },
                {
                    'title': _('Django "django-users" mailing list'),
                    'url': 'http://groups.google.com/group/django-users',
                    'external': True,
                },

            ]
        ))


class CustomAppIndexDashboard(AppIndexDashboard):
    """
    Custom app index dashboard for devcoop.
    """

    # we disable title because its redundant with the model list module
    title = ''

    def __init__(self, *args, **kwargs):
        AppIndexDashboard.__init__(self, *args, **kwargs)

        # append a model list module and a recent actions module
        self.children += [
            modules.ModelList(self.app_title, self.models),
            modules.RecentActions(
                _('Recent Actions'),
                include_list=self.get_app_content_types(),
                limit=5
            )
        ]

    def init_with_context(self, context):
        """
        Use this method if you need to access the request context.
        """
        return super(CustomAppIndexDashboard, self).init_with_context(context)



# -*- coding: utf-8 -*-
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as _
from django.template.loader import get_template
from django.template import Context
from coop_cms.settings import get_article_class
from django.conf import settings
from coop_bar.utils import make_link

from coop_cms.coop_bar_cfg import can_do
from coop_cms.coop_bar_cfg import cms_extra_js


#can_do teste editable=true et que l'objet est presetn dans le contexte et que l'utilisateur a le droit sur ce droit

can_edit_organization = can_do('can_edit',['organization']) # methode du model (doit s'appeller can_edit_organization) + objet du contexte

@can_edit_organization
def edit_org(request, context):
    org = context.get('organization')
    url = reverse('org_detail', args=[org.slug])
    return make_link(url, _(u'Edit organization'), 'fugue/store.png',classes=['icon'])

# 
# @can_post_event
# def post_event(request, context):
#     org = context.get('organization')
#     url = reverse('org_detail', args=[org.slug])
#     return make_link(url, _(u'Post new event'), 'fugue/calendar--plus.png.png',classes=['icon'])
# 


def load_commands(coop_bar):
    coop_bar.register([
        [edit_org,]#post_event]
    ])
    
    coop_bar.register_header(cms_extra_js)



# -*- coding:utf-8 -*-
from django.conf.urls.defaults import *

urlpatterns = patterns('',
    url(r'^(?P<uuid>[a-f0-9]{8}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{12})/$', 
        'coop.exchange.views.exchange_detail', 
        name="exchange_detail"),
)



# -*- coding:utf-8 -*-
from django.conf.urls.defaults import patterns, include, url

from django.contrib.auth import views as auth_views
from forms import ProfileForm

urlpatterns = patterns('',

    (r'^login/$', 'django.contrib.auth.views.login', {'template_name':'accounts/login.html'}),
    (r'^logout/$', 'django.contrib.auth.views.logout', {'template_name':'accounts/logout.html'}),
    
    # interception de la page edition de profil pour injecter notre formulaire
    (r'^edit/$', 'profiles.views.edit_profile', {'form_class': ProfileForm,}),
    
    url(r'^(?P<username>[_a-z]+)/$', 'profiles.views.profile_detail', name='profiles_profile_detail'),

    url(r'^password/change/$',auth_views.password_change,name='auth_password_change'),
    url(r'^password/change/done/$',auth_views.password_change_done,name='auth_password_change_done'),
    url(r'^password/reset/$',auth_views.password_reset,name='auth_password_reset'),
    url(r'^password/reset/confirm/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)/$',auth_views.password_reset_confirm,name='auth_password_reset_confirm'),
    url(r'^password/reset/complete/$',auth_views.password_reset_complete,name='auth_password_reset_complete'),
    url(r'^password/reset/done/$',auth_views.password_reset_done,name='auth_password_reset_done'),
    
    # sinon on délégue à l'app profiles
    (r'^', include('profiles.urls')),
    
  
)



# -*- coding:utf-8 -*-
from django.conf.urls.defaults import patterns, include, url
from django.views.generic.base import TemplateView, RedirectView
from django.conf import settings

class TextPlainView(TemplateView):
  def render_to_response(self, context, **kwargs):
    return super(TextPlainView, self).render_to_response(
      context, content_type='text/plain', **kwargs)

urlpatterns = patterns('',

    url(r'^$', 'coop.views.home', name="home"),
    (r'^org/', include('coop.org.urls')),
    (r'^comments/', include('django.contrib.comments.urls')),

    # a revoir
    (r'^perso/$', 'coop.person.views.perso'),
    (r'^membre/', include('coop.person.urls')),
    
    url(r'^robots\.txt$', TextPlainView.as_view(template_name='robots.txt')),
    url(r'^favicon\.ico$', RedirectView.as_view(url='/media/img/favicon.ico')),
    
)

if 'coop_tag' in settings.INSTALLED_APPS:
    urlpatterns += patterns('',
        url(r'^taggit_autosuggest/', include('taggit_autosuggest.urls')),
        )


# -*- coding:utf-8 -*-

from django.conf.urls.defaults import *
#from django.views.generic import ListView
#from coop.org.views import ISDetailView
#from coop_local.models import Organization

urlpatterns = patterns('',

    # (r'^$', ListView.as_view(
    #     model=Organization,
    #     context_object_name='liste_initiatives',
    #     template_name='initiative/initiative_list.html'
    #     )),

    url(r'^$', 'coop.org.views.org_list', name="org_list"),
    url(r'^global_map$', 'coop.org.views.global_map', name="org_global_map"),
    url(r'^(?P<slug>[\w-]+)/$', 'coop.org.views.org_detail_view', name="org_detail"),
    url(r'^(?P<slug>[\w-]+)/edit/$', 'coop.org.views.org_edit', name="org_edit"),
    
    url(r'^role/(?P<slug>[\w-]+)/$', 'coop.org.views.role_detail', name="role_detail"),
    
    #     
    # (r'^(?P<slug>\w+).html$', ISDetailView.as_view(
    #     model=Organization,
    #     #context_object_name='liste_initiatives'
    #     template_name = 'initiative_detail.html'
    #     
    #     )),    
        
)



# -*- coding:utf-8 -*-
from django.db import models
from extended_choices import Choices
from django_extensions.db import fields as exfields
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
import sorl
from django.core.exceptions import ValidationError
from django.conf import settings
from coop.models import URIModel

DISPLAY = Choices(
    ('PUBLIC',  1,  _(u'public information')),
    ('USERS',   2,  _(u'registered users')),
    ('MEMBER',  3,  _(u'members of an organization')),
    ('RELATED', 4,  _(u'related organizations members')),
    ('ORG',     5,  _(u'members of this organization')),
    ('ADMIN',   6,  _(u'administrators of this site')),
)

# will apply to contact numbers and other things
# TODO simplify it, see PPO Ontology

COMM_MEANS = Choices(
    ('MAIL',    8,  _(u'E-mail')),
    ('LAND',    1,  _(u'Landline phone')),
    ('GSM',     2,  _(u'Mobile phone')),
    ('FAX',     3,  _(u'Fax')),
    ('WEB',     9,  _(u'Secondary web site')),
    ('SKYPE',   4,  _(u'Skype')),
    ('TWITTER', 5,  _(u'Twitter')),
    ('RSS',     6,  _(u'RSS Feed')),
    ('VCAL',    7,  _(u'vCalendar')),
)


class BaseContact(models.Model):
    category = models.PositiveSmallIntegerField(_(u'Category'), 
                    choices=COMM_MEANS.CHOICES)
    content = models.CharField(_(u'content'), max_length=250)
    details = models.CharField(_(u'details'), blank=True, max_length=100)
    display = models.PositiveSmallIntegerField(_(u'Display'), 
                    choices=DISPLAY.CHOICES, default=DISPLAY.PUBLIC)
    created = exfields.CreationDateTimeField(_(u'created'), null=True)
    modified = exfields.ModificationDateTimeField(_(u'modified'), null=True)
    content_type = models.ForeignKey(ContentType, blank=True, null=True)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')

    class Meta:
        abstract = True
        ordering = ['category']
        verbose_name = _(u'Contact')
        verbose_name_plural = _(u'Contacts')

    def __unicode__(self):
        if self.content_object != None:
            return self.content + u' (' + self.content_object.__unicode__() + u')'
        else:    
            return self.content

    def clean(self):
        if self.category in [1, 2, 3]:
            import re
            phoneSplitRegex = re.compile(r"[\-\(\) \.\,]")         
            parts = phoneSplitRegex.split(self.content)
            num = ''.join(parts)
            if len(num) == 10:
                self.content = '.'.join((num[:2], num[2:4], num[4:6], num[6:8], num[8:]))
            else:
                raise ValidationError(_(u'Phone numbers must have 10 digits'))

    def save(self, *args, **kwargs):
        self.clean()
        super(BaseContact, self).save(*args, **kwargs) 


class BaseRole(models.Model):
    label = models.CharField(_(u'label'), max_length=60)
    slug = exfields.AutoSlugField(populate_from=('label'))

    class Meta:
        abstract = True
        ordering = ['label']

    def __unicode__(self):
        return unicode(self.label)

    def get_absolute_url(self):
        return reverse('role_detail', args=[self.slug])


RELATIONS = Choices(
    ('MEMBER',          1,  _(u' is member of ')),
    ('REG_SUPPLIER',    2,  _(u' has for regular supplier ')),
    ('OCC_SUPPLIER',    3,  _(u' has for casual supplier ')),
    ('SUPPORT',         4,  _(u' received technical support from ')),
    ('FUNDING',         5,  _(u' received financial support from ')),
)


class BaseRelation(models.Model):
    source = models.ForeignKey('coop_local.Organization', verbose_name=_(u'source organization'), related_name='source')
    target = models.ForeignKey('coop_local.Organization', verbose_name=_(u'target organization'), related_name='target')
    reltype = models.PositiveSmallIntegerField(_(u'Relation type'), choices=RELATIONS.CHOICES)
    created = exfields.CreationDateTimeField(_(u'created'), null=True)
    modified = exfields.ModificationDateTimeField(_(u'modified'), null=True)
    confirmed = models.BooleanField(default=False, verbose_name=_(u'confirmed by the target organization'))
    
    class Meta:
        abstract = True

    def __unicode__(self):    
        return u'"' + self.source.__unicode__() + u'"' + \
            unicode(RELATIONS.CHOICES_DICT[self.reltype]) + \
            u'"' + self.target.__unicode__() + u'".'
    '''
    def save(self):
        vérifier si la relation inverse existe
    '''
    

class BaseEngagement(models.Model):
    person = models.ForeignKey('coop_local.Person', verbose_name=_(u'person'), related_name='engagements') 
    organization = models.ForeignKey('coop_local.Organization', verbose_name=_(u'organization')) 
    role = models.ForeignKey('coop_local.Role', verbose_name=_(u'role'))
    role_detail = models.CharField(_(u'detailed role'), blank=True, max_length=100)
    created = exfields.CreationDateTimeField(_(u'created'), null=True)
    modified = exfields.ModificationDateTimeField(_(u'modified'), null=True)
    uri = models.CharField(_(u'main URI'), blank=True, null=True,
                            max_length=250, editable=False)  # FIXME : null=True incompatible with unique=True
    uuid = exfields.UUIDField(blank=True, null=True)  # FIXME : NULL=True for easier SQL import / equivalent in PGSQL default UUID ?
    org_admin = models.BooleanField(_(u'admin'), default=True)
    engagement_display = models.PositiveSmallIntegerField(_(u'Display'), choices=DISPLAY.CHOICES, default=DISPLAY.PUBLIC)
    
    class Meta:
        abstract = True
        verbose_name = _('Engagement')
        verbose_name_plural = _('Engagements')

    def __unicode__(self):
        return self.person.__unicode__()


class BaseOrganizationCategory(models.Model):
    label = models.CharField(blank=True, max_length=100)
    slug = exfields.AutoSlugField(populate_from=('label'))

    class Meta:
        abstract = True
        verbose_name = _(u'organization category')
        verbose_name_plural = _(u'organization categories')

    def __unicode__(self):
        return self.label

PREFLABEL = Choices(
    ('TITLE',   1,  _(u'Title')),
    ('ACRO',   2,  _(u'Acronym')),
)


class BaseOrganization(URIModel):
    title = models.CharField(_(u'title'), max_length=250)

    # acronym     = models.CharField(_(u'acronym'),max_length=100)
    # pref_label = models.PositiveSmallIntegerField(_(u'Preferred label'), choices=PREFLABEL.CHOICES, default=PREFLABEL.TITLE)
    
    subtitle = models.CharField(_(u'subtitle'), blank=True, null=True, 
                max_length=250,
                help_text=_(u'another name your organization is known by, or a tagline'))
    
    description = models.TextField(_(u'description'), blank=True, null=True)
    uri = models.CharField(_(u'main URI'), blank=True, null=True, 
                max_length=250, editable=False)
    logo = sorl.thumbnail.ImageField(upload_to='logos/', null=True, blank=True)
    relations = models.ManyToManyField('coop_local.Organization', 
                symmetrical=False, through='coop_local.Relation', 
                verbose_name=_(u'relations'))

    category = models.ManyToManyField('coop_local.OrganizationCategory', 
                blank=True, null=True, verbose_name=_(u'category'))

    members = models.ManyToManyField('coop_local.Person', 
                through='coop_local.Engagement', verbose_name=_(u'members'))
    
    contacts = generic.GenericRelation('coop_local.Contact')
    subs = generic.GenericRelation('coop_local.Subscription')

    # coop_geo must be loaded BEFORE coop_local
    if "coop_geo" in settings.INSTALLED_APPS:
        located = generic.GenericRelation('coop_geo.Located')
        area = generic.GenericRelation('coop_geo.AreaLink')
        
    birth = models.DateField(_(u'creation date'), null=True, blank=True)
    email = models.EmailField(_(u'global email'), blank=True, null=True)
    email_sha1 = models.CharField(_(u'email checksum'), 
            max_length=250, blank=True, null=True)  # TODO : do this in Postgre
    web = models.URLField(_(u'web site'), blank=True, null=True)

    pref_email = models.ForeignKey('coop_local.Contact', 
                verbose_name=_(u'preferred email'), 
                related_name="pref_email", null=True, blank=True)
    pref_phone = models.ForeignKey('coop_local.Contact', 
                verbose_name=_(u'preferred phone'), 
                related_name='pref_phone', null=True, blank=True)
    pref_address = models.ForeignKey('coop_geo.Location', 
                verbose_name=_(u'preferred postal address'), 
                related_name='pref_adress', null=True, blank=True)

    slug = exfields.AutoSlugField(populate_from='title', blank=True)
    created = exfields.CreationDateTimeField(_(u'created'), null=True)
    modified = exfields.ModificationDateTimeField(_(u'modified'), null=True)
    active = models.BooleanField(_(u'active'), default=True,)
    notes = models.TextField(_(u'notes'), blank=True)

    class Meta:
        abstract = True
        ordering = ['title']
        verbose_name = _(u'Organization')
        verbose_name_plural = _(u'Organizations')

    def __unicode__(self):
        return unicode(self.title)

    @property
    def uri_id(self):
        return self.slug

    uri_fragment = 'org'

    def can_edit_organization(self, user):
        return True

    if "coop_geo" in settings.INSTALLED_APPS:          
        def has_location(self):
            return self.located.all().count() > 0
        has_location.boolean = True    
        has_location.short_description = _(u'geo')
    
    def has_description(self):
        return self.description != None and len(self.description) > 20
    has_description.boolean = True    
    has_description.short_description = _(u'desc.')
    
    #@models.permalink
    def get_absolute_url(self):
        return reverse('org_detail', args=[self.slug])
        
    def get_tags(self):
        return self.tags.all()
        
    def get_relations(self):
        relations = {}
        relmap = RELATIONS.REVERTED_CHOICES_CONST_DICT
        
        for rel in self.source.all():
            reltype = str('OUT_' + relmap[rel.reltype])  # me => others
            relations[reltype] = []
            relations[reltype].append(rel.target)
        for rel in self.target.all():
            reltype = str('IN_' + relmap[rel.reltype])  # others said this
            if reltype not in relations:
                relations[reltype] = []
            if rel.confirmed:  # which one are confirmed by both parts
                relations[reltype].append(rel.source)
        return relations
        
    def local_uri(self):
        return ('http://dev.credis.org:8000/org/' + self.slug + '/')

    def save(self, *args, **kwargs):
        # Set default values for preferred email, phone and postal address
        if self.pref_phone == None:
            fixes = self.contacts.filter(category=1)
            if fixes.count() == 1:
                self.pref_phone = fixes[0]
        if self.pref_email == None:
            orgmails = self.contacts.filter(category=8)
            if orgmails.count() == 1:
                self.pref_email = orgmails[0]
        if 'coop_geo' in settings.INSTALLED_APPS:
            if self.pref_address == None:
                locations = self.located.all()  # should we have a "main venue" ?
                if locations.count() == 1:
                    self.pref_address = locations[0].location

        if self.email != '':
            import hashlib
            m = hashlib.sha1()
            m.update(self.email)
            self.email_sha1 = m.hexdigest()
        super(BaseOrganization, self).save(*args, **kwargs)  




"""
@author: Łukasz Kidziński
"""
from django.contrib.auth.models import User
from django.core.validators import email_re

class EmailBackend:
    """
    Authenticate with e-mail.

    Use the  e-mail, and password
    
    Should work with django 1.3
    """

    supports_object_permissions = False
    supports_anonymous_user = False
    supports_inactive_user = False

    def authenticate(self, username=None, password=None):
        if email_re.search(username):
            try:
                user = User.objects.get(email=username)
            except User.DoesNotExist:
                return None
        else:
            #We have a non-email address username we should try username
            try:
                user = User.objects.get(username=username)
            except User.DoesNotExist:
                return None
        if user.check_password(password):
            return user
        return None

    def get_user(self, user_id):
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None



from django import template
from urlparse import urlparse, urlunparse

register = template.Library()

@register.simple_tag
def favicon(url):
    parsed_url = urlparse(url)

    return urlunparse((parsed_url[0], parsed_url[1],
                       u'favicon.ico', parsed_url[3],
                       parsed_url[4], parsed_url[5]))


# -*- coding:utf-8 -*-

import rdflib, rdfalchemy
from rdflib import Namespace, plugin
from rdflib import ConjunctiveGraph as Graph

FOAF = Namespace("http://xmlns.com/foaf/0.1/")

graph = Graph()
graph.parse("http://danbri.org/foaf.rdf")

# tester les données

# plugin.register(
#     "sparql", rdflib.query.Processor,
#     "rdfextras.sparql.processor", "Processor")
# plugin.register(
#     "sparql", rdflib.query.Result,
#     "rdfextras.sparql.query", "SPARQLQueryResult") 

# for row in g.query('SELECT ?a ?b ?aname ?bname WHERE { ?a foaf:knows ?b . OPTIONAL { ?a foaf:name ?aname . ?b foaf:name ?bname . } }',initNs={'foaf':FOAF}):
#     print "friend: %s" % row[1]


class Person(rdfalchemy.rdfSubject):
  rdfalchemy.rdfSubject.db = graph
  rdf_type = FOAF.Person
  name = rdfalchemy.rdfSingle(FOAF.name)
  mbox = rdfalchemy.rdfSingle(FOAF.mbox)


for p in Person.ClassInstances():
    print p.name




# -*- coding:utf-8 -*-

import floppyforms as forms
from coop_cms.forms import ArticleForm as CmsArticleForm
from coop_cms.settings import get_article_class
from djaloha.widgets import AlohaInput

class ArticleForm(CmsArticleForm):
    class Meta:
        model = get_article_class()
        fields = ('title', 'content', 'logo', 'author')
        widgets = {
            'title': AlohaInput(),
            'content': AlohaInput(),
        }


# -*- coding:utf-8 -*-
from django.contrib.auth.models import User
from django.db import models
from django_extensions.db import fields as exfields
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse
from django.template.defaultfilters import slugify
from django.contrib.contenttypes import generic
from django.contrib.sites.models import Site
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from coop.models import URIModel

if "coop_geo" in settings.INSTALLED_APPS:
    from coop_geo.models import Location


class BasePersonCategory(models.Model):
    label = models.CharField(blank=True, max_length=100)
    slug = exfields.AutoSlugField(populate_from=('label'))

    class Meta:
        abstract = True
        verbose_name = _(u'Person category')
        verbose_name_plural = _(u'Person categories')
        
    def __unicode__(self):
        return self.label    

from coop.org.models import DISPLAY


class BasePerson(URIModel):
    user = models.OneToOneField(User, blank=True, null=True, unique=True, verbose_name=_(u'django user'), editable=False)
    username = models.CharField(blank=True, max_length=100, unique=True)    
    #pour D2RQ et poura voir des URI clean meme pour des non-users
    
    category = models.ManyToManyField('coop_local.PersonCategory', blank=True, null=True, verbose_name=_(u'category'))
    last_name = models.CharField(_(u'last name'), max_length=100)
    first_name = models.CharField(_(u'first name'), max_length=100, null=True, blank=True)
    contact = generic.GenericRelation('coop_local.Contact')
    email = models.EmailField(_(u'personal email'), blank=True)
    email_sha1 = models.CharField(_(u'email checksum'), max_length=250, blank=True, null=True)
    created = exfields.CreationDateTimeField(_(u'created'), null=True)
    modified = exfields.ModificationDateTimeField(_(u'modified'), null=True)
    uri = models.CharField(_(u'main URI'), blank=True, max_length=250, null=True)   # FIXME : NULL just for easier importing of data
                                                                                    # could be a lambda function property ?
    notes = models.TextField(_(u'notes'), blank=True, null=True)
    structure = models.CharField(blank=True, max_length=100)

    subs = generic.GenericRelation('coop_local.Subscription')

    if "coop_geo" in settings.INSTALLED_APPS:
        location = models.ForeignKey(Location, null=True, blank=True, verbose_name=_(u'location'))
        location_display = models.PositiveSmallIntegerField(_(u'Display'), choices=DISPLAY.CHOICES, default=DISPLAY.USERS)

    class Meta:
        abstract = True
        verbose_name = _(u'Person')
        verbose_name_plural = _(u'Persons')
        
    def __unicode__(self):
        return unicode('%s %s' % (self.first_name, self.last_name))
        
    @models.permalink
    def get_absolute_url(self):
        return reverse('profiles_profile_detail', args=[self.username])
        #return ('profiles_profile_detail', (), {'username': self.username})
        
    def has_user_account(self):
        return (self.user != None)
    has_user_account.boolean = True    
    has_user_account.short_description = _(u'django account')
    
    def has_role(self):
        return (self.engagements.count() > 0)
    has_role.boolean = True    
    has_role.short_description = _(u'has organization')
    
    def engagements(self):
        eng = []
        for e in self.engagement_set.all():
            eng.append({'initiative': e.initiative, 'role': e.role})
        return eng

    @property
    def uri_id(self):
        return self.username
    uri_fragment = 'person'

    def save(self, *args, **kwargs):
        if self.email != '':
            import hashlib
            m = hashlib.sha1()
            m.update(self.email)
            self.email_sha1 = m.hexdigest()

        # create username slug if not set
        if self.username == '':
            newname = slugify(self.first_name).replace('-', '_') + '.' + \
                        slugify(self.last_name).replace('-', '_')
            self.username = newname
        # synchronize fields with django User model
        if self.user:
            chg = False
            for field in ('username', 'first_name', 'last_name', 'email'):
                if(getattr(self.user, field) != getattr(self, field)):
                    setattr(self.user, field, getattr(self, field))
                    chg = True
            if(chg):
                self.user.save()
        super(BasePerson, self).save(*args, **kwargs)    




# -*- coding:utf-8 -*-
from django.db import models
from django_extensions.db import fields as exfields
from django.utils.translation import ugettext_lazy as _
from extended_choices import Choices
import datetime
from django.core.urlresolvers import reverse
from decimal import Decimal
from django.conf import settings
from coop.models import URIModel

if "coop_geo" in settings.INSTALLED_APPS:
    from coop_geo.models import Area, Location


MODALITIES = Choices(
    ('GIFT',    1,  _(u'Gift')),
    ('TROC',    2,  _(u'Free exchange')),
    ('CURR',    3,  _(u'Monetary exchange')),
)
UNITS = Choices(
    ('EURO',    1,  _(u'€')),
    ('SELH',    2,  _(u'Hours')),
    ('PEZ',     3,  _(u'PEZ')),    
)


class BasePaymentModality(models.Model):
    exchange = models.ForeignKey('coop_local.Exchange', verbose_name=_(u'exchange'), related_name='modalities')
    modality = models.PositiveSmallIntegerField(_(u'exchange type'), blank=True,
                                                choices=MODALITIES.CHOICES,
                                                default=MODALITIES.CURR)
    amount = models.DecimalField(_(u'amount'), max_digits=12, decimal_places=2, default=Decimal(0.00), blank=True)
    unit = models.PositiveSmallIntegerField(_(u'unit'), blank=True, null=True, choices=UNITS.CHOICES)
    
    def __unicode__(self):
        if(self.modality in [1, 2]):
            return unicode(MODALITIES.CHOICES_DICT[self.modality])
        elif(self.modality == 3 and self.amount > 0 and not self.unit == None):
            return unicode("%s %s" % (self.amount, unicode(UNITS.CHOICES_DICT[self.unit])))
        elif(self.modality == 3):
            return unicode(_(u'Price unknown'))

    def save(self, *args, **kwargs):
        if not self.modality == 3:
            self.amount = Decimal(0.00)
            self.unit = None
        super(BasePaymentModality, self).save(*args, **kwargs) 
                
    class Meta:
        abstract = True
        verbose_name = _(u'Payment modality')
        verbose_name_plural = _(u'Payment modalities')


EXCHANGE = Choices(
    ('P_OFFER',   1,  _(u'Product Offer')),
    ('S_OFFER',   2,  _(u'Service Offer')),
    ('P_NEED',    3,  _(u'Product Need')),
    ('S_NEED',    4,  _(u'Service Need')),
    ('MUTU',    7,  _(u'Mutualization')),  # 3
    ('COOP',    8,  _(u'Cooperation, partnership')),  # 5
    ('QA',      9,  _(u'Question')),  # 6
)


class BaseProduct(URIModel):
    title = models.CharField(_('title'), blank=True, max_length=250)
    slug = exfields.AutoSlugField(populate_from='title')
    description = models.TextField(_(u'description'), blank=True)
    organization = models.ForeignKey('coop_local.Organization', blank=True, null=True, 
                                        verbose_name='publisher', related_name='products')
    created = exfields.CreationDateTimeField(_(u'created'), null=True)
    modified = exfields.ModificationDateTimeField(_(u'modified'), null=True)
    uri = models.CharField(_(u'main URI'), blank=True, max_length=250, editable=False)
    publisher_uri = models.CharField(_(u'publisher URI'), blank=True, max_length=200, editable=False)
    
    @property
    def uri_id(self):
        return self.id
    uri_fragment = 'product'

    def __unicode__(self):
        return self.title

    class Meta:
        abstract = True
        verbose_name = _(u'Product')
        verbose_name_plural = _(u'Products')


class BaseExchange(URIModel):
    title = models.CharField(_('title'), max_length=250)
    description = models.TextField(_(u'description'), blank=True)
    organization = models.ForeignKey('coop_local.Organization', blank=True, null=True, 
                            verbose_name=_('publisher'), related_name='exchanges')
    person = models.ForeignKey('coop_local.Person', blank=True, null=True, editable=False, verbose_name=_(u'person'))
    etype = models.PositiveSmallIntegerField(_(u'exchange type'), choices=EXCHANGE.CHOICES)
    permanent = models.BooleanField(_(u'permanent'), default=True)
    expiration = models.DateField(_(u'expiration'), blank=True, null=True)
    slug = exfields.AutoSlugField(populate_from='title')
    created = exfields.CreationDateTimeField(_(u'created'), null=True)
    modified = exfields.ModificationDateTimeField(_(u'modified'), null=True)
    products = models.ManyToManyField('coop_local.Product', verbose_name=_('linked products'))
    
    uri = models.CharField(_(u'main URI'), blank=True, max_length=250, editable=False)
    author_uri = models.CharField(_(u'author URI'), blank=True, max_length=200, editable=False)
    publisher_uri = models.CharField(_(u'publisher URI'), blank=True, max_length=200, editable=False)

    uuid = exfields.UUIDField()  # nécessaire pour URI ?
    # coop_geo must be loaded BEFORE coop_local
    if "coop_geo" in settings.INSTALLED_APPS:
        location = models.ForeignKey(Location, null=True, blank=True, verbose_name=_(u'location'))
        area = models.ForeignKey(Area, null=True, blank=True, verbose_name=_(u'area'))    
    
    def __unicode__(self):
        return unicode(self.title)

    def get_absolute_url(self):
        return reverse('annonce_detail', args=[self.uuid])
        
    #TODO assign the record to the person editing it (form public) and provide an A-C choice in admin

    @property
    def uri_id(self):
        return self.uuid
    uri_fragment = 'exchange'

    class Meta:
        abstract = True
        verbose_name = _(u'Exchange')
        verbose_name_plural = _(u'Exchanges')
       

class BaseTransaction(models.Model):
    origin = models.ForeignKey('coop_local.Exchange', related_name='origin', verbose_name=_(u'origin'))
    destination = models.ForeignKey('coop_local.Exchange', related_name='destination', verbose_name=_(u'destination'))
    origin_org = models.ForeignKey('coop_local.Organization', related_name='contrats_vente', verbose_name=_(u'vendor'), blank=True, null=True)
    destination_org = models.ForeignKey('coop_local.Organization', related_name='contrats_achat', verbose_name=_(u'buyer'), blank=True, null=True)
    title = models.CharField(_('title'), blank=True, max_length=250)
    description = models.TextField(_(u'description'), blank=True)
    created = exfields.CreationDateTimeField(_(u'created'), null=True)
    modified = exfields.ModificationDateTimeField(_(u'modified'), null=True)
    uuid = exfields.UUIDField()  # nécessaire pour URI ?
    
    def __unicode__(self):
        return self.title
        
    class Meta:
        abstract = True
        verbose_name = _(u'Transaction')
        verbose_name_plural = _(u'Transactions')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

try:
    from setuptools import setup, find_packages
except ImportError:
    import ez_setup
    ez_setup.use_setuptools()
    from setuptools import setup, find_packages

VERSION = __import__('coop').__version__

import os
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name='django-coop',
    version = VERSION,
    description='A basis for a cooperative organization directory, with people, organization, offers.',
    packages=[  'coop',
                'coop.exchange',
                'coop.link',
                'coop.org',
                'coop.person',
                'coop.templatetags',
                'coop.utils',
                ],
    include_package_data=True,
    author='Cooperative Quinode',
    author_email='contact@quinode.fr',
    license='BSD',
    zip_safe=False,
    install_requires = ['south==0.7.3',
                        'sorl-thumbnail==11.09',
                        'django-extensions==0.6',
                        'django-admin-tools==0.4.1',
                        #'django-haystack==1.2.6',
                        #'djangoembed==0.1.1', # plutot via github
                        ],
    long_description = open('README.rst').read(),
    url = 'https://github.com/quinode/django-coop/',
    download_url = 'https://github.com/quinode/django-coop/tarball/master',
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
        'Natural Language :: English',
        'Natural Language :: French',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
    ],

)




# -*- coding:utf-8 -*-
from django.db import models
from django.contrib.sites.models import Site


class URIModel(models.Model):
    """
    To use this model as a basis for your own abstract model, you need to have
    a 'uri_id' property set and a string for the model type URL representation :

    @property
    def uri_id(self):
        return self.username # example with username

    uri_fragment = 'org'    

    the uri_fragment can then be overriden in a real model derived from your abstract model

    """
    class Meta:
        abstract = True

    def init_uri(self):
        return 'http://' + str(Site.objects.get_current().domain) + \
                    '/id/'+ str(self.uri_fragment) + \
                    '/' + str(getattr(self, 'uri_id')) + '/'

    def save(self, *args, **kwargs):
        # create / update URI
        if self.uri != self.init_uri():
            self.uri = self.init_uri()
        super(URIModel, self).save(*args, **kwargs)



#
#    Autocomplete feature for admin panel
#
#    Most of the code has been written by Jannis Leidel, then updated a bit
#    for django_extensions, finally reassembled and extended by Mikele Pasin. 
# 
#    http://jannisleidel.com/2008/11/autocomplete-form-widget-foreignkey-model-fields/
#    http://code.google.com/p/django-command-extensions/
#    http://magicrebirth.wordpress.com/
#
#    to_string_function, Satchmo adaptation and some comments added by emes
#    (Michal Salaban)
#




#  ==============
#  HOW-TO
#  ==============

# 1.Put this file somewhere in your application folder

# 2.Add the 'autocomplete' folder with the media files to your usual media folder

# 3.Add the 'admin/autocomplete' folder to your templates folder

# 4.Add the extrafilters.py file in the templatetags directory of your application (or just add its contents to 
#   your custom template tags if you already have some).
#  All is needed is the 'cut' filter, for making the code used in the inline-autocomplete form javascript friendly

# 5. When defining your models admin, import the relevant admin and use it:
#  .....
# from myproject.mypackage.autocomplete_admin import FkAutocompleteAdmin
#  .....
#  .....
# class Admin (FkAutocompleteAdmin):
#   related_search_fields = { 'person': ('name',)}
#  .....


#  ==============
# TROUBLE SHOOTING:
#  ==============

# ** sometimes things don't work cause you have to add 'from django.conf.urls.defaults import *' to the modules where you use the autocomplete 
# ** if you're using the inline-autocompletion, make sure that the main admin class the inline belong to is a subclass FkAutocompleteAdmin
# ** you may need to hack it a bit to make it work for you - it's been done in a rush!


#  ==============
# the code now....
#  ==============


from django import forms
from django.conf import settings
from django.utils.safestring import mark_safe
from django.utils.text import truncate_words
from django.template.loader import render_to_string
from django.contrib.admin.widgets import ForeignKeyRawIdWidget
import operator
from django.http import HttpResponse, HttpResponseNotFound
from django.contrib import admin
from django.db import models
from django.db.models.query import QuerySet
from django.utils.encoding import smart_str
from django.utils.translation import ugettext as _
from django.utils.text import get_text_list
# added by mikele
from django.conf.urls.defaults import *




class FkSearchInput(ForeignKeyRawIdWidget):
    """
    A Widget for displaying ForeignKeys in an autocomplete search input 
    instead in a <select> box.
    """

    # Set in subclass to render the widget with a different template
    widget_template = None
    # Set this to the patch of the search view
    search_path = '../foreignkey_autocomplete/'

    class Media:
        css = {
            'all': ('autocomplete/css/jquery.autocomplete.css',)
        }
        js = (
            #'autocomplete/js/jquery.js',
            'autocomplete/js/jquery.bgiframe.min.js',
            'autocomplete/js/jquery.ajaxQueue.js',
            'autocomplete/js/jquery.autocomplete.js',
        )

    def label_for_value(self, value):
        key = self.rel.get_related_field().name
        obj = self.rel.to._default_manager.get(**{key: value})
        return truncate_words(obj, 14)

    def __init__(self, rel, search_fields, attrs=None):
        self.search_fields = search_fields
        super(FkSearchInput, self).__init__(rel, attrs)

    def render(self, name, value, attrs=None):
        if attrs is None:
            attrs = {}
        output = [super(FkSearchInput, self).render(name, value, attrs)]
        opts = self.rel.to._meta
        app_label = opts.app_label
        model_name = opts.object_name.lower()
        related_url = '../../../%s/%s/' % (app_label, model_name)
        params = self.url_parameters()
        if params:
            url = '?' + '&amp;'.join(['%s=%s' % (k, v) for k, v in params.items()])
        else:
            url = ''
        if not attrs.has_key('class'):
            attrs['class'] = 'vForeignKeyRawIdAdminField'
        # Call the TextInput render method directly to have more control
        output = [forms.TextInput.render(self, name, value, attrs)]
        if value:
            label = self.label_for_value(value)
        else:
            label = u''
        context = {
            'url': url,
            'related_url': related_url,
            'admin_media_prefix': settings.ADMIN_MEDIA_PREFIX,
            'search_path': self.search_path,
            'search_fields': ','.join(self.search_fields),
            'model_name': model_name,
            'app_label': app_label,
            'label': label,
            'name': name,
        }


        output.append(render_to_string(self.widget_template or (
            '%s/%s/fk_searchinput.html' % (app_label, model_name),
            '%s/fk_searchinput.html' % app_label,
            'admin/autocomplete/fk_searchinput.html',
        ), context))
        output.reverse()
        return mark_safe(u''.join(output))






class NoLookupsForeignKeySearchInput(ForeignKeyRawIdWidget):
    """
    A Widget for displaying ForeignKeys in an autocomplete search input 
    instead in a <select> box.
    """

    # Set in subclass to render the widget with a different template
    widget_template = None
    # Set this to the patch of the search view
    search_path = '../foreignkey_autocomplete/'

    class Media:
        css = {
            'all': ('autocomplete/css/jquery.autocomplete.css',)
        }
        js = (
            #'autocomplete/js/jquery.js',
            'autocomplete/js/jquery.bgiframe.min.js',
            'autocomplete/js/jquery.ajaxQueue.js',
            'autocomplete/js/jquery.autocomplete.js',
        )

    def label_for_value(self, value):
        key = self.rel.get_related_field().name
        obj = self.rel.to._default_manager.get(**{key: value})
        return truncate_words(obj, 14)

    def __init__(self, rel, search_fields, attrs=None):
        self.search_fields = search_fields
        super(NoLookupsForeignKeySearchInput, self).__init__(rel, attrs)

    def render(self, name, value, attrs=None):
        if attrs is None:
            attrs = {}
        output = [super(NoLookupsForeignKeySearchInput, self).render(name, value, attrs)]
        opts = self.rel.to._meta
        app_label = opts.app_label
        model_name = opts.object_name.lower()
        related_url = '../../../%s/%s/' % (app_label, model_name)
        params = self.url_parameters()
        if params:
            url = '?' + '&amp;'.join(['%s=%s' % (k, v) for k, v in params.items()])
        else:
            url = ''
        if not attrs.has_key('class'):
            attrs['class'] = 'vForeignKeyRawIdAdminField'
        # Call the TextInput render method directly to have more control
        output = [forms.TextInput.render(self, name, value, attrs)]
        if value:
            label = self.label_for_value(value)
        else:
            label = u''
        context = {
            'url': url,
            'related_url': related_url,
            'admin_media_prefix': settings.ADMIN_MEDIA_PREFIX,
            'search_path': self.search_path,
            'search_fields': ','.join(self.search_fields),
            'model_name': model_name,
            'app_label': app_label,
            'label': label,
            'name': name,
        }

# tried to change widget_template but it needs a subclass (of what?) not a string

        output.append(render_to_string(self.widget_template or (
            '%s/%s/foreignkey_searchinput.html' % (app_label, model_name),
            '%s/foreignkey_searchinput.html' % app_label,
            'admin/autocomplete/nolookups_foreignkey_searchinput.html',
        ), context))
        output.reverse()
        return mark_safe(u''.join(output))



class InlineSearchInput(ForeignKeyRawIdWidget):
    """
    A Widget for displaying ForeignKeys in an autocomplete search input 
    instead in a <select> box.
    """

    # Set in subclass to render the widget with a different template
    widget_template = None
    # Set this to the patch of the search view
    search_path = '../foreignkey_autocomplete/'

    class Media:
        css = {
            'all': ('autocomplete/css/jquery.autocomplete.css',)
        }
        js = (
            #'autocomplete/js/jquery.js',
            'autocomplete/js/jquery.bgiframe.min.js',
            'autocomplete/js/jquery.ajaxQueue.js',
            'autocomplete/js/jquery.autocomplete.js',
        )

    def label_for_value(self, value):
        key = self.rel.get_related_field().name
        obj = self.rel.to._default_manager.get(**{key: value})
        return truncate_words(obj, 14)

    def __init__(self, rel, search_fields, attrs=None):
        self.search_fields = search_fields
        super(InlineSearchInput, self).__init__(rel, attrs)

    def render(self, name, value, attrs=None):
        if attrs is None:
            attrs = {}
        output = [super(InlineSearchInput, self).render(name, value, attrs)]
        opts = self.rel.to._meta
        app_label = opts.app_label
        model_name = opts.object_name.lower()
        related_url = '../../../%s/%s/' % (app_label, model_name)
        params = self.url_parameters()
        if params:
            url = '?' + '&amp;'.join(['%s=%s' % (k, v) for k, v in params.items()])
        else:
            url = ''
        if not attrs.has_key('class'):
            attrs['class'] = 'vForeignKeyRawIdAdminField'
        # Call the TextInput render method directly to have more control
        output = [forms.TextInput.render(self, name, value, attrs)]
        if value:
            label = self.label_for_value(value)
        else:
            label = u''
        context = {
            'url': url,
            'related_url': related_url,
            'admin_media_prefix': settings.ADMIN_MEDIA_PREFIX,
            'search_path': self.search_path,
            'search_fields': ','.join(self.search_fields),
            'model_name': model_name,
            'app_label': app_label,
            'label': label,
            'name': name,
        }

# tried to change widget_template but it needs a subclass (of what?) not a string

        output.append(render_to_string(self.widget_template or (
            '%s/%s/inline_searchinput.html' % (app_label, model_name),
            '%s/inline_searchinput.html' % app_label,
            'admin/autocomplete/inline_searchinput.html',
        ), context))
        output.reverse()
        return mark_safe(u''.join(output))









# ====================================
# ====================================
#  standard FK autocomplete
#  ===================================
#  ===================================




class FkAutocompleteAdmin(admin.ModelAdmin):
    """ 
    Admin class for models using the autocomplete feature.

    There are two additional fields:
       - related_search_fields: defines fields of managed model that
         have to be represented by autocomplete input, together with
         a list of target model fields that are searched for
         input string, e.g.:

         related_search_fields = {
            'author': ('first_name', 'email'),
         }

       - related_string_functions: contains optional functions which
         take target model instance as only argument and return string
         representation. By default __unicode__() method of target
         object is used.
    """

    related_search_fields = {}
    related_string_functions = {}

    # def __call__(self, request, url):
    #     if url is None:
    #         pass
    #     elif url == 'foreignkey_autocomplete':
    #         return self.foreignkey_autocomplete(request)
    #     return super(ForeignKeyAutocompleteAdmin, self).__call__(request, url)

    def get_urls(self):
        urls = super(FkAutocompleteAdmin,self).get_urls()
        search_url = patterns('',
            (r'^foreignkey_autocomplete/$', self.admin_site.admin_view(self.foreignkey_autocomplete))
        )
        return search_url + urls

    def foreignkey_autocomplete(self, request):
        """
        Searches in the fields of the given related model and returns the 
        result as a simple string to be used by the jQuery Autocomplete plugin
        """
        query = request.GET.get('q', None)
        app_label = request.GET.get('app_label', None)
        model_name = request.GET.get('model_name', None)
        search_fields = request.GET.get('search_fields', None)
        object_pk = request.GET.get('object_pk', None)
        try:
            to_string_function = self.related_string_functions[model_name]
        except KeyError:
            to_string_function = lambda x: x.__unicode__()
        if search_fields and app_label and model_name and (query or object_pk):
            def construct_search(field_name):
                # use different lookup methods depending on the notation
                if field_name.startswith('^'):
                    return "%s__istartswith" % field_name[1:]
                elif field_name.startswith('='):
                    return "%s__iexact" % field_name[1:]
                elif field_name.startswith('@'):
                    return "%s__search" % field_name[1:]
                else:
                    return "%s__icontains" % field_name
            model = models.get_model(app_label, model_name)
            queryset = model._default_manager.all()
            data = ''
            if query:
                for bit in query.split():
                    or_queries = [models.Q(**{construct_search(
                        smart_str(field_name)): smart_str(bit)})
                            for field_name in search_fields.split(',')]
                    other_qs = QuerySet(model)
                    other_qs.dup_select_related(queryset)
                    other_qs = other_qs.filter(reduce(operator.or_, or_queries))
                    queryset = queryset & other_qs
                data = ''.join([u'%s|%s\n' % (
                    to_string_function(f), f.pk) for f in queryset])
            elif object_pk:
                try:
                    obj = queryset.get(pk=object_pk)
                except:
                    pass
                else:
                    data = to_string_function(obj)
            return HttpResponse(data)
        return HttpResponseNotFound()

    def get_help_text(self, field_name, model_name):
        searchable_fields = self.related_search_fields.get(field_name, None)
        if searchable_fields:
            help_kwargs = {
                'model_name': model_name,
                'field_list': get_text_list(searchable_fields, _('and')),
            }
            return _('Use the left field to do %(model_name)s lookups in the fields %(field_list)s.') % help_kwargs
        return ''


# this method gets called when creating the formfields - probably this is what you need to extend
#  in the replicated version of ForeignKeyAutocompleteAdmin

    def formfield_for_dbfield(self, db_field, **kwargs):
        """
        Overrides the default widget for Foreignkey fields if they are
        specified in the related_search_fields class attribute.
        """
        if (isinstance(db_field, models.ForeignKey) and 
            db_field.name in self.related_search_fields):
            model_name = db_field.rel.to._meta.object_name
            help_text = self.get_help_text(db_field.name, model_name)
            if kwargs.get('help_text'):
                help_text = u'%s %s' % (kwargs['help_text'], help_text)
            kwargs['widget'] = FkSearchInput(db_field.rel,
                                    self.related_search_fields[db_field.name])
            kwargs['help_text'] = help_text
        return super(FkAutocompleteAdmin,
            self).formfield_for_dbfield(db_field, **kwargs)




# ====================================
# ====================================
#  an autocomplete that doesn't display the raw_id links
#  ===================================
#  ===================================



class NoLookupsForeignKeyAutocompleteAdmin(admin.ModelAdmin):
    """ 
        In certain cases you do not want to have the usual raw_id lenses for related items lookup.
        Code mostly as above, changes only the template that renders it.
    """

    related_search_fields = {}
    related_string_functions = {}

    # def __call__(self, request, url):
    #     if url is None:
    #         pass
    #     elif url == 'foreignkey_autocomplete':
    #         return self.foreignkey_autocomplete(request)
    #     return super(ForeignKeyAutocompleteAdmin, self).__call__(request, url)

    def get_urls(self):
        urls = super(NoLookupsForeignKeyAutocompleteAdmin,self).get_urls()
        search_url = patterns('',
            (r'^foreignkey_autocomplete/$', self.admin_site.admin_view(self.foreignkey_autocomplete))
        )
        return search_url + urls

    def foreignkey_autocomplete(self, request):
        """
        Searches in the fields of the given related model and returns the 
        result as a simple string to be used by the jQuery Autocomplete plugin
        """
        query = request.GET.get('q', None)
        app_label = request.GET.get('app_label', None)
        model_name = request.GET.get('model_name', None)
        search_fields = request.GET.get('search_fields', None)
        object_pk = request.GET.get('object_pk', None)
        try:
            to_string_function = self.related_string_functions[model_name]
        except KeyError:
            to_string_function = lambda x: x.__unicode__()
        if search_fields and app_label and model_name and (query or object_pk):
            def construct_search(field_name):
                # use different lookup methods depending on the notation
                if field_name.startswith('^'):
                    return "%s__istartswith" % field_name[1:]
                elif field_name.startswith('='):
                    return "%s__iexact" % field_name[1:]
                elif field_name.startswith('@'):
                    return "%s__search" % field_name[1:]
                else:
                    return "%s__icontains" % field_name
            model = models.get_model(app_label, model_name)
            queryset = model._default_manager.all()
            data = ''
            if query:
                for bit in query.split():
                    or_queries = [models.Q(**{construct_search(
                        smart_str(field_name)): smart_str(bit)})
                            for field_name in search_fields.split(',')]
                    other_qs = QuerySet(model)
                    other_qs.dup_select_related(queryset)
                    other_qs = other_qs.filter(reduce(operator.or_, or_queries))
                    queryset = queryset & other_qs
                data = ''.join([u'%s|%s\n' % (
                    to_string_function(f), f.pk) for f in queryset])
            elif object_pk:
                try:
                    obj = queryset.get(pk=object_pk)
                except:
                    pass
                else:
                    data = to_string_function(obj)
            return HttpResponse(data)
        return HttpResponseNotFound()

    def get_help_text(self, field_name, model_name):
        searchable_fields = self.related_search_fields.get(field_name, None)
        if searchable_fields:
            help_kwargs = {
                'model_name': model_name,
                'field_list': get_text_list(searchable_fields, _('and')),
            }
            return _('Use the left field to do %(model_name)s lookups in the fields %(field_list)s.') % help_kwargs
        return ''


# this method gets called when creating the formfields - probably this is what you need to extend
#  in the replicated version of ForeignKeyAutocompleteAdmin

    def formfield_for_dbfield(self, db_field, **kwargs):
        """
        Overrides the default widget for Foreignkey fields if they are
        specified in the related_search_fields class attribute.
        """
        if (isinstance(db_field, models.ForeignKey) and 
            db_field.name in self.related_search_fields):
            model_name = db_field.rel.to._meta.object_name
            help_text = self.get_help_text(db_field.name, model_name)
            if kwargs.get('help_text'):
                help_text = u'%s %s' % (kwargs['help_text'], help_text)
            kwargs['widget'] = NoLookupsForeignKeySearchInput(db_field.rel,
                                    self.related_search_fields[db_field.name])
            kwargs['help_text'] = help_text
        return super(NoLookupsForeignKeyAutocompleteAdmin,
            self).formfield_for_dbfield(db_field, **kwargs)






#  ======================================
#  ======================================
#  an autocomplete for INLINES
#  ======================================
#  ======================================




class InlineAutocompleteAdmin(admin.TabularInline):
    """ 
    Admin class for models using the autocomplete feature in inlines.

    At the moment, this autocomplete works only if the admin of the model including the inline-admin is
    itself a subclass of an autocomplete Admin (e.g., ForeignKeyAutocompleteAdmin)
    
    """

    related_search_fields = {}
    related_string_functions = {}

    # def __call__(self, request, url):
    #     if url is None:
    #         pass
    #     elif url == 'foreignkey_autocomplete':
    #         return self.foreignkey_autocomplete(request)
    #     return super(ForeignKeyAutocompleteAdmin, self).__call__(request, url)

    def get_urls(self):
        urls = super(InlineAutocompleteAdmin,self).get_urls()
        search_url = patterns('',
            (r'^foreignkey_autocomplete/$', self.admin_site.admin_view(self.foreignkey_autocomplete))
        )
        return search_url + urls

    def foreignkey_autocomplete(self, request):
        """
        Searches in the fields of the given related model and returns the 
        result as a simple string to be used by the jQuery Autocomplete plugin
        """
        query = request.GET.get('q', None)
        app_label = request.GET.get('app_label', None)
        model_name = request.GET.get('model_name', None)
        search_fields = request.GET.get('search_fields', None)
        object_pk = request.GET.get('object_pk', None)
        try:
            to_string_function = self.related_string_functions[model_name]
        except KeyError:
            to_string_function = lambda x: x.__unicode__()
        if search_fields and app_label and model_name and (query or object_pk):
            def construct_search(field_name):
                # use different lookup methods depending on the notation
                if field_name.startswith('^'):
                    return "%s__istartswith" % field_name[1:]
                elif field_name.startswith('='):
                    return "%s__iexact" % field_name[1:]
                elif field_name.startswith('@'):
                    return "%s__search" % field_name[1:]
                else:
                    return "%s__icontains" % field_name
            model = models.get_model(app_label, model_name)
            queryset = model._default_manager.all()
            data = ''
            if query:
                for bit in query.split():
                    or_queries = [models.Q(**{construct_search(
                        smart_str(field_name)): smart_str(bit)})
                            for field_name in search_fields.split(',')]
                    other_qs = QuerySet(model)
                    other_qs.dup_select_related(queryset)
                    other_qs = other_qs.filter(reduce(operator.or_, or_queries))
                    queryset = queryset & other_qs
                data = ''.join([u'%s|%s\n' % (
                    to_string_function(f), f.pk) for f in queryset])
            elif object_pk:
                try:
                    obj = queryset.get(pk=object_pk)
                except:
                    pass
                else:
                    data = to_string_function(obj)
            return HttpResponse(data)
        return HttpResponseNotFound()

    def get_help_text(self, field_name, model_name):
        searchable_fields = self.related_search_fields.get(field_name, None)
        if searchable_fields:
            help_kwargs = {
                'model_name': model_name,
                'field_list': get_text_list(searchable_fields, _('and')),
            }
            return _('Use the left field to do %(model_name)s lookups in the fields %(field_list)s.') % help_kwargs
        return ''


# this method gets called when creating the formfields - probably this is what you need to extend
#  in the replicated version of ForeignKeyAutocompleteAdmin

    def formfield_for_dbfield(self, db_field, **kwargs):
        """
        Overrides the default widget for Foreignkey fields if they are
        specified in the related_search_fields class attribute.
        """
        if (isinstance(db_field, models.ForeignKey) and 
            db_field.name in self.related_search_fields):
            model_name = db_field.rel.to._meta.object_name
            help_text = self.get_help_text(db_field.name, model_name)
            if kwargs.get('help_text'):
                help_text = u'%s %s' % (kwargs['help_text'], help_text)
            kwargs['widget'] = InlineSearchInput(db_field.rel,
                                    self.related_search_fields[db_field.name])
            kwargs['help_text'] = help_text
        return super(InlineAutocompleteAdmin,
            self).formfield_for_dbfield(db_field, **kwargs)





#  ===========
# using the autocomplete admin with other custom admin classes: 
# just mix and match as you like.... 
#  e.g. in my case I used it with the admin for trees provided by FeinCms:

try:
    from feincms.admin import editor
    #  just merging the effects of the two classes..
    class AutocompleteTreeEditor(FkAutocompleteAdmin, editor.TreeEditor):
        def __init__(self, *args, **kwargs):
            super(AutocompleteTreeEditor, self).__init__(*args, **kwargs)
except ImportError:
    pass
#  ===========











# -*- coding:utf-8 -*-
from django.db import models
from django_extensions.db import fields as exfields
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic


class BaseMailingList(models.Model):
    name = models.CharField(_('name'), max_length=50, unique=True)
    email = models.EmailField(_('Newsletter email'))
    main_url = models.URLField(_('List URL'))

    class Meta:
        verbose_name = _('mailing list')
        verbose_name_plural = _('mailing lists')
        abstract = True

    def __unicode__(self):
        return u'%s' % (self.name)


class BaseSubscription(models.Model):
    mailing_list = models.ForeignKey('coop_local.MailingList',
                                        related_name='subs')
    created = exfields.CreationDateTimeField(_(u'created'), null=True)
    modified = exfields.ModificationDateTimeField(_(u'modified'), null=True)
    # subscription options (HTML, text ...)
    email = models.EmailField(_('subscribed email'), default='')

    # things which are suscribed
    content_type = models.ForeignKey(ContentType, blank=True, null=True)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')

    class Meta:
        verbose_name = _('mailing list subscription')
        verbose_name_plural = _('mailing list subscriptions')
        abstract = True

    def __unicode__(self):
        return _(u'subscription to ') + unicode(self.mailing_list)
